#!/usr/bin/python
# encoding=utf8


# 演示如何使用BoxSizer
# 
# 开发环境：
#   python: 2.7
#   wxPython: 4.1.0

import sys
sys.path.append("../..")

import xmlui
import wx

class MainController(xmlui.Controller):
    def __init__(self):
        pass

    def after_load(self):
        pass

    def OnClickSubmit(self, evt):
    	s = u"用户名:"+self.ui_name.GetValue()+"\n"
    	s += u"密码:"+self.ui_pass.GetValue()+"\n"
        wx.MessageBox(s)

def main():
    app = xmlui.load_wx("wx_boxsize.xml", [MainController])
    app.MainLoop()

if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: UTF-8 -*-

import xml.etree.ElementTree as ET

# 负责解析xml的逻辑过程
class DocParser:
    # xml_root: 要解析的xml节点
    # handle_classes(dict(tag,class)): 注册遇到tag时的处理类
    # defaultHandleClass(): 遇到没有注册的tag时的处理类
    # controllers(dict(name,class)): xml中的controller
    def __init__(self, xml_root, handle_classes, defaultHandleClass, controllers):
        self.xml_root = xml_root
        self.handle_class = handle_classes
        self.defaultHandleClass = defaultHandleClass
        self.controllers = controllers or {}

    def parse(self):
        return self.recursive_parse(None, self.xml_root)

    def recursive_parse(self, handle_parent, xml_element):
        HandleClass = self.handle_class.get(xml_element.tag, None) or self.defaultHandleClass
        handle_obj = HandleClass()

        controller_name = xml_element.get("controller", "").strip()
        if controller_name:
            new_controller = self.controllers[controller_name]()
        else:
            new_controller = None

        controller = new_controller
        if not controller and handle_parent:
            controller = handle_parent.controller
        handle_obj.init(self, handle_parent, xml_element, controller)

        handle_obj.handle()

        name = xml_element.get("name", "").strip()
        if name and handle_obj.get_result() and handle_parent and not hasattr(handle_parent.controller, name):
            setattr(handle_parent.controller or handle_parent.get_result(), name, handle_obj.get_result())

        handle_obj.handle_over()

        if new_controller:
            setattr(handle_obj.get_result(), "controller", new_controller)
            new_controller.node = handle_obj.get_result()
            new_controller.after_load()

        return handle_obj

# 处理一个tag及其属性的基类
class HandleTagBase:
    # parser(DocParser):
    # parent(HandleTagBase): 父处理节点
    # xml_element(): 要解析的节点
    # controller: 这个节点由哪个controller控制
    def init(self, parser, parent, xml_element, controller):
        self.parser = parser
        self.parent = parent
        self.xml_element = xml_element
        self.controller = controller

        # 处理结果
        # 
        # 处理后得到的ui节点
        self.ui = None
        # 处理后得到的非ui节点
        self.custom = None
        # 直接子节点(HandleTagBase)
        self.children = []

    def get_result(self):
        return self.ui or self.custom

    # 向上回溯，直到找到一个ui节点，包括自身
    def get_latest_ui(self):
        handle_obj = self
        while handle_obj and not handle_obj.ui:
            handle_obj = handle_obj.parent
        if handle_obj:
            return handle_obj.ui

    def handle(self):
        self.handle_self()
        self.handle_children()

    def handle_children(self):
        for child in self.xml_element:
            child_handle_obj = self.parser.recursive_parse(self, child)
            self.children.append(child_handle_obj)
            self.after_handle_child(child_handle_obj)

    # 子类应该重写这个类，在这个方法内部设置self.ui或self.custom节点
    def handle_self(self):
        pass

    # 每次解析完一个子节点回调一次
    def after_handle_child(self, child_handle_obj):
        pass

    # 整个节点解析完的回调
    def handle_over(self):
        pass

# 从字符串解析出python值
# attrValue: 
def convert_attr_value(attrValue, globals_dict):
    if not attrValue.strip():
        return attrValue
    else:
        try:
            return eval(attrValue, globals_dict)
        except:
            return attrValue


#!/usr/bin/python
# -*- coding: UTF-8 -*-

import wx

from . import xml_ui_tool

def get_all_handle_class():
    return {k:v for k,v in globals().items() if type(v)==type or type(v).__name__=="classobj"}

def convert_attr_value(value):
    return xml_ui_tool.convert_attr_value(value, {"wx":wx})

class HandleCommonTag(xml_ui_tool.HandleTagBase):
    def handle_self(self):
        iscallfunc = self.check_call_func()
        if iscallfunc:
            return

        self.custom = self.create_custom()
        if self.custom:
            self.apply_all_attrs()
            return

        self.ui = self.create_ui()
        if self.ui:
            self.apply_all_attrs()
            return

    # 优先假设xml_element.tag是一个可执行函数
    def check_call_func(self):
        func = getattr(self.parent, self.xml_element.tag, None)
        if not func:
            func = getattr(self, self.xml_element.tag, None)
        if not func and self.parent:
            func = getattr(self.parent.get_result(), self.xml_element.tag, None)

        if not callable(func):
            return False

        if not self.xml_element.text or not self.xml_element.text.strip():
            func()
        else:
            params = xml_ui_tool.convert_attr_value(self.xml_element.text, {"wx":wx})
            if isinstance(params, tuple):
                func(*params)
            else:
                func(params)

        return True

    def create_custom(self):
        return None

    # 尝试创建ui节点
    def create_ui(self):
        wxClass = getattr(wx, self.xml_element.tag, None)
        if not wxClass:
            return None
        
        args = {}
        self.pick_constructor_arg(args, "id")
        self.pick_constructor_arg(args, "title")
        self.pick_constructor_arg(args, "pos")
        self.pick_constructor_arg(args, "size")
        self.pick_constructor_arg(args, "style")
        self.pick_constructor_arg(args, "label")
        
        ui = wxClass(self.get_latest_ui(), **args)

        return ui

    def apply_all_attrs(self):
        for attrName, attrValue in self.xml_element.items():
            func = getattr(self, "apply_attr_"+attrName, None)
            if func:
                func(xml_ui_tool.convert_attr_value(attrValue, {"wx":wx}))
                continue

            func = getattr(self.get_result(), attrName, None)
            if callable(func):
                func(xml_ui_tool.convert_attr_value(attrValue, {"wx":wx}))

    def pick_constructor_arg(self, args, name):
        attr = self.xml_element.get(name)
        if attr and attr.strip():
            args[name] = xml_ui_tool.convert_attr_value(attr, {"wx":wx})

    def apply_attr_title(self, value):
        self.get_result().SetTitle(value)

    def apply_attr_onclick(self, value):
        if isinstance(self.get_result(), wx.Button):
            bindtype = wx.EVT_BUTTON
            bindfunc = getattr(self.controller, value.strip(), None)
            if bindfunc:
                self.get_latest_evt_handler().Bind(bindtype, bindfunc, None)
        elif isinstance(self.get_result(), wx.MenuItem):
            bindtype = wx.EVT_MENU
            bindfunc = getattr(self.controller, value.strip(), None)
            if bindfunc:
                self.get_latest_evt_handler().Bind(bindtype, bindfunc, id=self.custom.GetId())

    def get_latest_evt_handler(self):
        handle_obj = self
        while handle_obj and not isinstance(handle_obj.get_result(), wx.EvtHandler):
            handle_obj = handle_obj.parent
        if handle_obj:
            return handle_obj.get_result()

    def handle_over(self):
        if type(self.ui) in [wx.Window, wx.Panel, wx.Frame] and not self.ui.GetSizer():
            self.add_default_layout()

    def add_default_layout(self):
        ui_children = filter(lambda c: c.ui, self.children)
        if len(ui_children)==0: return

        one_child = len(ui_children)==1

        orient = wx.VERTICAL if self.xml_element.get("sizer_orient")=="v" else wx.HORIZONTAL

        sizer = wx.BoxSizer(orient)
        for child in self.children:
            if child.ui:
                proportion = 1 if one_child else int(child.xml_element.get("sizer_proportion", "0"))
                flags = wx.EXPAND if one_child else convert_attr_value(child.xml_element.get("sizer_flags", "0"))
                sizer.Add(child.ui, proportion, flags)
        self.ui.SetSizer(sizer)

class App(xml_ui_tool.HandleTagBase):
    def __init__(self):
        self.main_frame = None
    def handle_self(self):
        self.custom = wx.App()
    def after_handle_child(self, child_handle_obj):
        if self.main_frame:
            return

        if child_handle_obj.ui:
            self.main_frame = child_handle_obj.ui
            self.main_frame.Show()

class BoxSizer(xml_ui_tool.HandleTagBase):
    def handle_self(self):
        orient = wx.HORIZONTAL
        if self.xml_element.get("orient", None)=="v":
            orient = wx.VERTICAL

        self.custom = wx.BoxSizer(orient)

        self.proportion = None
        if self.xml_element.get("proportion", "").strip():
            self.proportion = xml_ui_tool.convert_attr_value(self.xml_element.get("proportion"), {"wx":wx})
        if isinstance(self.proportion, int):
            self.proportion = [self.proportion]

        self.flags = None
        if self.xml_element.get("flags", None):
            self.flags = xml_ui_tool.convert_attr_value(self.xml_element.get("flags"), {"wx":wx})
        if isinstance(self.flags, int):
            self.flags = [self.flags]

    def after_handle_child(self, child_handle_obj):
        if child_handle_obj.ui:
            prop = self.proportion[len(self.children)-1] if self.proportion and len(self.proportion)>=len(self.children) else 0
            if not self.flags:
                flag = 0
            elif len(self.flags)>=len(self.children):
                flag = self.flags[len(self.children)-1]
            else:
                flag = self.flags[-1]
            self.custom.Add(child_handle_obj.ui, prop, flag)

    def handle_over(self):
        self.parent.ui.SetSizer(self.custom)
        self.custom.Fit(self.parent.ui)

class SplitterWindow(HandleCommonTag):
    def SetSashPosition(self, attrValue):
        p1, p2 = self.ui.GetChildren()
        if self.ui.GetSplitMode()==wx.SPLIT_HORIZONTAL:
            self.ui.SplitHorizontally(p1, p2, attrValue)
        elif self.ui.GetSplitMode()==wx.SPLIT_VERTICAL:
            self.ui.SplitVertically(p1, p2, attrValue)

class Bind(HandleCommonTag):
    def handle_self(self):
        bindtype = xml_ui_tool.convert_attr_value(self.xml_element.get("type"), {"wx":wx})
        bindfunc = getattr(self.controller, self.xml_element.text.strip(), None)
        if bindfunc:
            self.get_latest_evt_handler().Bind(bindtype, bindfunc, None)

class MenuBar(HandleCommonTag):
    def create_custom(self):
        return wx.MenuBar()
    def after_handle_child(self, child_handle_obj):
        self.custom.Append(child_handle_obj.get_result(), child_handle_obj.label)
    def handle_over(self):
        if self.parent.ui:
            self.parent.ui.SetMenuBar(self.custom)

class Menu(HandleCommonTag):
    def create_custom(self):
        return wx.Menu()
    def after_handle_child(self, child_handle_obj):
        if isinstance(child_handle_obj, Menu):
            self.get_result().AppendSubMenu(child_handle_obj.get_result(), child_handle_obj.label)
        else:
            self.get_result().Append(child_handle_obj.get_result())
    def apply_attr_label(self, value):
        self.label = value

class MenuItem(HandleCommonTag):
    def create_custom(self):
        id = int(self.xml_element.get("id", "-1"))
        text = self.xml_element.get("label",None) or self.xml_element.get("text",None)
        helpString = self.xml_element.get("helpString","")
        kind = convert_attr_value(self.xml_element.get("kind", "wx.ITEM_NORMAL"))
        return wx.MenuItem(None, id, text, helpString, kind)












#!/usr/bin/python
# -*- coding: UTF-8 -*-

from . import xml_ui_tool
import xml.etree.ElementTree as ET

class Controller:
    def __init__(self):
        # node: 存储对应tag的处理结果
        self.node = None

    def after_load(self):
        pass

def load_wx(file_or_xmlstring, controllers):
    if "<" in file_or_xmlstring or ">" in file_or_xmlstring:
        xml_root = ET.fromstring(file_or_xmlstring)
    else:
        xml_doc = ET.parse(file_or_xmlstring)
        xml_root = xml_doc.getroot()

    from . import xml_wx

    handle_class = xml_wx.get_all_handle_class()

    if isinstance(controllers, list):
        controllers = {clas.__name__:clas for clas in controllers}
    elif controllers:
        controllers = {controllers.__name__:controllers}
    else:
        controllers = {}

    parser = xml_ui_tool.DocParser(xml_root, handle_class, xml_wx.HandleCommonTag, controllers)
    handle_app = parser.parse()
    return handle_app.get_result()



#!/usr/bin/python
# encoding=utf8


# 演示如何添加菜单
# 
# 开发环境：
#   python: 2.7
#   wxPython: 4.1.0

import sys
sys.path.append("../..")

import xmlui
import wx

class MainController(xmlui.Controller):
    def __init__(self):
        pass

    def after_load(self):
        pass

    def OnMenuItem1(self, evt):
    	wx.MessageBox("OnMenuItem1")
    def OnMenuItem2(self, evt):
    	self.node.PopupMenu(self.ui_popup)

    def OnMenuPopup(self, evt):
        wx.MessageBox("popup")

def main():
    app = xmlui.load_wx("wx_menu.xml", [MainController])
    app.MainLoop()

if __name__ == '__main__':
    main()



#!/usr/bin/python
# encoding=utf8


# 展示如何加载多个窗口
# 
# 开发环境：
#   python: 2.7
#   wxPython: 4.1.0

import sys
sys.path.append("../..")

import xmlui
import wx

class MainController(xmlui.Controller):
    def __init__(self):
        pass

    def after_load(self):
        pass

    def OnClickButton(self, evt):
        self.ui_mydlg.ShowModal()

def main():
    app = xmlui.load_wx("wx_multiwindow.xml", [MainController])
    app.MainLoop()

if __name__ == '__main__':
    main()
